/******************************************************************************
*                                                                             *
* C Source: Linked_List                                                       *
*                                                                             *
* File:                                                                       *
*    lnkd_lst.c                                                               *
*                                                                             *
* Description:                                                                *
*    This set of routines manipulate linked list data structures. Here, the   *
*    implementation of a linked list consists of a header structure which     *
*    should always remain at the same address. This header has a single field *
*    which points to the first node structure of the linked list.  Each node  *
*    in the linked list points to the previous and next nodes, and the data   *
*    for the node. Note that the data is not actually a part of the node      *
*    structure, but is merely pointed to by the node. (see lnkd_lst.h.)       *
*                                                                             *
* Module List:                                                                *
*    Create_Empty_Linked_List                                                 *
*       Allocate memory for the header of a linked list.                      *
*    Delete_Linked_List                                                       *
*       Delete linked list from memory, but not the data it points to.        *
*    Delete_Linked_List_Data                                                  *
*       Delete linked list from memory, AND the data it points to.            *
*    Delete_Linked_List_Node                                                  *
*       Delete a node from a linked list, but not the data it contains.       *
*    Delete_Linked_List_Nodes                                                 *
*       Delete all nodes of a linked list from memory, but not the data       *
*       contain. Note, this still leaves the header of the linked list        *
*       intact.                                                               *
*    Find_Linked_List_Data                                                    *
*       Find data in linked list matching input data.                         *
*    Find_Linked_List_Node                                                    *
*       Find node in linked list whose data matches input data.               *
*    Insert_Linked_List_Data                                                  *
*       Insert data via a new node at the top of a linked list.               *
*    Linked_List_Is_Empty                                                     *
*       Returns TRUE if linked list is empty, otherwise, FALSE.               *
*                                                                             *
* History:                                                                    *
*    Created    : 02-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
*    Updated    : 22-JUN-94 by J. Doggett,                                    *
*       Updated Insert_Linked_List_Data function.                             *
*    Redelivered: 30-NOV-94 by J. Doggett,                                    *
*       GetImage Version 1.1.                                                 *
*                                                                             *
* Copyright (c) 1993, 1994, Association of Universities for Research in       *
* Astronomy, Inc.  All Rights Reserved.                                       *
* Produced under National Aeronautics and Space Administration grant          *
* NAG W-2166.                                                                 *
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "lnkd_lst.h"

#include "config.h"
#include "generic.h"

/******************************************************************************
* Module:                                                                     *
*    Create_Empty_Linked_List                                                 *
*                                                                             *
* File:                                                                       *
*    lnkd_lst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Create an empty linked list by allocating memory for the header, and     *
*    point the header to null.                                                *
*                                                                             *
* Return Value:                                                               *
*    LL: The newly created linked list.                                       *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       none                                                                  *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 02-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/****************************************
* Create_Empty_Linked_List declarations *
****************************************/
LL Create_Empty_Linked_List()
{
   LL List; /* New linked list */

/*************************************************
* Create_Empty_Linked_List begin executable code *
*************************************************/
   List = ( LL )Allocate_Memory( sizeof( *List ) );
   List->Top = NULL;

   return List;
}
/***********************************************
* Create_Empty_Linked_List end executable code *
***********************************************/

/******************************************************************************
*                                                                             *
* Module:                                                                     *
*    Delete_Linked_List                                                       *
*                                                                             *
* File:                                                                       *
*    lnkd_lst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Delete a linked list from memory, but not the data it points to.         *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       List                                                                  *
*          LL, Reference                                                      *
*             The linked list which we're deleting the node.                  *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 02-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/**********************************
* Delete_Linked_List declarations *
**********************************/
void Delete_Linked_List( List )
LL   List;
{
/*******************************************
* Delete_Linked_List begin executable code *
*******************************************/
   if ( List == NULL )
      return;

   Delete_Linked_List_Nodes( List->Top );

   free( List );
   return;
}
/*****************************************
* Delete_Linked_List end executable code *
*****************************************/

/******************************************************************************
* Module:                                                                     *
*    Delete_Linked_List_Data                                                  *
*                                                                             *
* File:                                                                       *
*    lnkd_lst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Delete a linked list from memory, and all the data it contains.          *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       List                                                                  *
*          LL, Reference                                                      *
*             The linked list which we're deleting the node.                  *
*       Delete                                                                *
*          void(*)(), Reference                                               *
*             A function used to delete the data pointed by the linked        *
*             list. It must have one argument that accepts the linked list    *
*             data passed by reference.                                       *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 02-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/***************************************
* Delete_Linked_List_Data declarations *
***************************************/
void   Delete_Linked_List_Data( List, Delete )
LL     List     ;
void (*Delete)();
{
  LL_NODE Node; /* Node of list we're deleting data from */

/*************************************************
* Delete_Linked_List_Data begin executable code  *
*************************************************/
   if ( List == NULL )
      return;

   Node = List->Top;
   while( Node != NULL )
   {
      Delete( Node->Data );
      Node = Node->Next;
   }
   Delete_Linked_List_Nodes( List->Top );

   free( List );

   return;

/****************************
* Delete_Linked_List page 2 *
****************************/

}
/*****************************************
* Delete_Linked_List end executable code *
*****************************************/

/******************************************************************************
* Module:                                                                     *
*    Delete_Linked_List_Node                                                  *
*                                                                             *
* File:                                                                       *
*    lnkd_lst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Delete a node from a linked list, but not the data it points to.         *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       List                                                                  *
*          LL, Reference                                                      *
*             The linked list from which we're deleting the node.             *
*       Node                                                                  *
*          LL_NODE, Reference                                                 *
*             The node we're deleting from the linked list.                   *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 02-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/***************************************
* Delete_Linked_List_Node declarations *
***************************************/
void    Delete_Linked_List_Node( List, Node )
LL      List;
LL_NODE Node;
{
/************************************************
* Delete_Linked_List_Node begin executable code *
************************************************/
   if ( Node->Previous == NULL )
      List->Top = Node->Next;
   else
      Node->Previous->Next = Node->Next;

   if ( Node->Next != NULL )
      Node->Next->Previous = Node->Previous;

   free( Node );
   return;
}
/**********************************************
* Delete_Linked_List_Node end executable code *
**********************************************/

/******************************************************************************
* Module:                                                                     *
*    Delete_Linked_List_Nodes                                                 *
*                                                                             *
* File:                                                                       *
*    lnkd_lst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Delete all nodes of a linked list from memory, but not the header nor    *
*    the data the nodes points to.                                            *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       List                                                                  *
*          LL, Reference                                                      *
*             The linked list which we're deleting the node.                  *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 02-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/****************************************
* Delete_Linked_List_Nodes declarations *
****************************************/
void    Delete_Linked_List_Nodes( Top_Node )
LL_NODE Top_Node;
{
/*************************************************
* Delete_Linked_List_Nodes begin executable code *
*************************************************/
   if ( Top_Node == NULL )
      return;

   Delete_Linked_List_Nodes( Top_Node->Next );

   free( Top_Node );
   return;
}
/***********************************************
* Delete_Linked_List_Nodes end executable code *
***********************************************/

/******************************************************************************
* Module:                                                                     *
*    Find_Linked_List_Data                                                    *
*                                                                             *
* File:                                                                       *
*    lnkd_lst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Returns data in a linked list that matches input data. The input data    *
*    for matching can be any data with in the linked list data, for instance, *
*    the data in the linked list maybe structures. The input data matched     *
*    against one of the fields. A pointer to the first structure in the       *
*    linked list matching the input field data will be returned. The only     *
*    catch is that the comparing function knows what to do.                   *
*                                                                             *
* Return Value:                                                               *
*    void*: Pointer to the data in the linked list matched to the input data. *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       List                                                                  *
*          LL, Reference                                                      *
*             The linked list from which we're looking for data.              *
*       Data                                                                  *
*          void*, Reference                                                   *
*             The input data we're going to match against.                    *
*       Compare                                                               *
*          int(*)(), Reference                                                *
*             Function used to compare input data against data in the list.   *
*             This function must have two arguments. This first argument must *
*             accept data from the linked list passed as void*. The second    *
*             must accept the input match data passed as void*. It must       *
*             return zero if the data is matched, and a non-zero number if    *
*             not.  It's a good idea to have it return a negative number if   *
*             its first argument is "less" than its second argument, and      *
*             return a positive number if the first is greater than the       *
*             second.                                                         *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 02-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/*************************************
* Find_Linked_List_Data declarations *
*************************************/
void  *Find_Linked_List_Data( List, Data, Compare   )
LL     List      ;
void  *Data      ;
int  (*Compare)();
{
   LL_NODE Node_with_Data; /* Node we found matching input data */




/*******************************
* Find_Linked_List_Data page 2 *
*******************************/

/**********************************************
* Find_Linked_List_Data begin executable code *
**********************************************/
   Node_with_Data = Find_Linked_List_Node( List->Top, Data, Compare );

   if ( Node_with_Data == NULL )
      return ( void* )NULL;

   else
      return Node_with_Data->Data;
}
/********************************************
* Find_Linked_List_Data end executable code *
********************************************/

/******************************************************************************
* Module:                                                                     *
*    Find_Linked_List_Node                                                    *
*                                                                             *
* File:                                                                       *
*    lnkd_lst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Return first node in a linked list starting from a given node whose data *
*    matches input data. The input data for matching can be any data within   *
*    the linked list data, for instance, the data in the linked list maybe    *
*    structures. The input data matched against one of the fields.  A pointer *
*    to the first structure in the linked list matching the input field data  *
*    will be returned. The only catch is that the comparing function knows    *
*    what to do. Note: this function is recursive.                            *
*                                                                             *
* Return Value:                                                               *
*    LL_NODE: Pointer to the node matching whose data matches the input data. *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       First_Node                                                            *
*          LL_NODE, Reference                                                 *
*             Node in the link list to start searching from.                  *
*          void*, Reference                                                   *
*             The input data we're going to match against.                    *
*       Compare                                                               *
*          int(*)(), Reference                                                *
*          int(*)(), Reference                                                *
*             Function used to compare input data against data in the list.   *
*             This function must have two arguments. This first argument must *
*             accept data from the linked list passed as void*. The second    *
*             must accept the input match data passed as void*. It must       *
*             return zero if the data is matched, and a non-zero number if    *
*             not.  It's a good idea to have it return a negative number if   *
*             its first argument is "less" than its second argument, and      *
*             return a positive number if the first is greater than the       *
*             second.                                                         *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 02-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/*************************************
* Find_Linked_List_Node declarations *
*************************************/
LL_NODE   Find_Linked_List_Node( First_Node, Data, Compare )
LL_NODE   First_Node;
void     *Data      ;
int     (*Compare)();
{
/**********************************************
* Find_Linked_List_Node begin executable code *
**********************************************/

/*******************************
* Find_Linked_List_Node page 2 *
*******************************/

   if ( First_Node == NULL )
      return ( LL_NODE )NULL;

   else if ( ( *Compare )( First_Node->Data, Data ) == 0 )
      return First_Node;

   else
      return Find_Linked_List_Node( First_Node->Next, Data, Compare );
}
/********************************************
* Find_Linked_List_Node end executable code *
********************************************/

/******************************************************************************
* Module:                                                                     *
*    Insert_Linked_List_Data                                                  *
*                                                                             *
* File:                                                                       *
*    lnkd_lst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Insert data at the top of a linked list.                                 *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       List                                                                  *
*          LL, Reference                                                      *
*             The linked list which we're deleting the node.                  *
*       Data                                                                  *
*          void*, Reference                                                   *
*             The data we're going to insert into the linked list.            *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 02-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
*    Updated    : 22-JUN-94 by J. Doggett,                                    *
*       Now resets Previous pointer of original Top node to the new node.     *
*    Redelivered: 30-NOV-94 by J. Doggett,                                    *
*       GetImage Version 1.1.                                                 *
******************************************************************************/
/***************************************
* Insert_Linked_List_Data declarations *
***************************************/
void  Insert_Linked_List_Data( List, Data )
LL    List;
void *Data;
{
   LL_NODE New_Node; /* New linked list node for the data */

/*************************************************
* Insert_Linked_List_Data begin executable code  *
*************************************************/
   New_Node = ( LL_NODE )Allocate_Memory( sizeof( *New_Node ) );


   New_Node->Data     = Data     ;
   New_Node->Next     = List->Top;
   New_Node->Previous = NULL     ;

   List->Top          = New_Node ;

   if (New_Node->Next != NULL)
      New_Node->Next->Previous = New_Node;

/*********************************
* Insert_Linked_List_Data page 2 *
*********************************/

   return;
}
/**********************************************
* Insert_Linked_List_Data end executable code *
**********************************************/

/******************************************************************************
* Module:                                                                     *
*    Linked_List_Is_Empty                                                     *
*                                                                             *
* File:                                                                       *
*    lnkd_lst.c                                                               *
*                                                                             *
* Description:                                                                *
*   Returns TRUE if linked list is empty, ie, List->Top is NULL. Othewise,    *
*   returns FALSE.                                                            *
*                                                                             *
*                                                                             *
* Return Value:                                                               *
*    BOOL: TRUE if linked list is empty.                                      *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       List                                                                  *
*          LL, Reference                                                      *
*             The linked list which we're deleting the node.                  *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 17-DEC-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/************************************
* Linked_List_Is_Empty declarations *
************************************/
BOOL Linked_List_Is_Empty( List )
LL   List;
{
/**********************************************
* Linked_List_Is_Empty begin executable code  *
**********************************************/

   if ( List->Top == NULL )
      return TRUE;
   else
      return FALSE;
}
/********************************************
* Linked_List_Is_Empty end executable code  *
********************************************/
